@@ -95,6 +95,106 @@ class Sbml4j(object):
         return files_dict
 
 
+    def getEntityInfo(self, geneSymbol):
+        self.checkSyncStatus()
+        baseUrl = "{}/entityInfo?".format(self._configuration.url)
+        if isinstance(geneSymbol, list):
+            geneSymbolParam = ",".join(geneSymbol)
+        else:
+            geneSymbolParam = geneSymbol
+        args_dict = {'geneSymbol': geneSymbolParam}
+        
+        # encode the arguments
+        encoded_args = urlencode(args_dict)
+        urlString = baseUrl + encoded_args
+        response = self._pm.request("GET", urlString,
+                                   headers=self._configuration.headers)
+        
+        if response.status > 399:
+            if not 'reason' in response.headers.keys():
+                raise Exception("Unknown Error fetching resource: HttpStatus: {}; Header of response: {}".format(response.status, response.headers))
+            else:
+                raise Exception("Could not fetch resource. Reason: {}".format(response.headers['reason']))
+        else:
+            return json.loads(response.data.decode('utf-8'))
+    
+    
+    def getIdMap(self, symbol, separator=",", idSystem=None):
+        self.checkSyncStatus()
+        baseUrl = "{}/idMap?".format(self._configuration.url)
+        
+        if isinstance(symbol, list):
+            symbolParam = separator.join(symbol)
+        else:
+            symbolParam = symbol
+        args_dict = {'symbol': symbolParam, 'separator': separator}
+        
+        if idSystem != None:
+            args_dict['idSystem'] = idSystem
+        
+        
+        # encode the arguments
+        encoded_args = urlencode(args_dict)
+        urlString = baseUrl + encoded_args
+        response = self._pm.request("GET", urlString,
+                                   headers=self._configuration.headers)
+        
+        if response.status > 399:
+            if not 'reason' in response.headers.keys():
+                raise Exception("Unknown Error fetching resource: HttpStatus: {}; Header of response: {}".format(response.status, response.headers))
+            else:
+                raise Exception("Could not fetch resource. Reason: {}".format(response.headers['reason']))
+        else:
+            return json.loads(response.data.decode('utf-8'))
+    ######################################## GraphML methods #####################################################
+    
+    def uploadGraphML(self, graphMLFiles, parentUUID=None, networkname=None, doPrefixName=None, doSuffixName=None):
+        self.checkSyncStatus()
+        baseUrl = "{}/graphml?".format(self._configuration.url)
+        # required arguments
+        args_dict = {}
+        # optional arguments
+        if parentUUID != None:
+            args_dict['parentUUID'] = parentUUID
+        if networkname != None:
+            args_dict['networkname'] = networkname
+        if doPrefixName != None:
+            args_dict['prefixName'] = doPrefixName
+        if doSuffixName != None:
+            args_dict['suffixName'] = doSuffixName
+    
+        urlString = baseUrl
+        if len(args_dict) > 0:
+            # encode the arguments
+            encoded_args = urlencode(args_dict)
+            urlString = urlString + encoded_args
+    
+        if not isinstance(graphMLFiles, list):
+            graphMLFiles = [graphMLFiles]
+            
+        files_dict = {}
+        for file in graphMLFiles:
+            with open(file) as graphml:
+                response = self._pm.request(
+                    "POST",
+                    urlString,
+                    fields={
+                        'files': (os.path.basename(graphml.name), graphml.read())
+                    },
+                    headers = self._configuration.headers
+                )
+                if response.status > 399:
+                    if not 'reason' in response.headers.keys():
+                        raise Exception("Unknown Error fetching resource: HttpStatus: {}; Header of response: {}".format(response.status, response.headers))
+                    else:
+                        raise Exception("Could not create resource. Reason: {}".format(response.headers['reason']))
+                else:
+                    networkInventoryItemList = json.loads(response.data.decode('utf-8'))
+                    files_dict[file]=networkInventoryItemList[0]
+        self._configuration.isInSync=False
+        return json.loads(response.data.decode('utf-8'))
+
+
     ######################################## Pathway methods #####################################################
 
     def pathwayList(self, hideCollections=None):
@@ -617,3 +717,67 @@ class Sbml4j(object):
                return annotatedNetworkDict
         else:
             raise Exception("No annotation data found. Cannot annotate network.")
+
+
+    # Add provenanceAnnotation to network
+    def addProvenanceToNetwork(self, uuid, name, jsonBody):
+        self.checkSyncStatus()
+        
+        # requestBody (needs to be json, and we simply use the dumps method to convert the provided data into json)
+        if not isinstance(jsonBody , dict):
+            raise Exception("jsonBody method parameter needs to be a dictionary type.")
+        
+        baseUrl = "{}/networks/{}/prov?".format(
+                    self._configuration.url,
+                    uuid
+                    )
+        # required arguments
+        args_dict = {'name': name}
+        
+        # encode the arguments
+        encoded_args = urlencode(args_dict)
+        urlString = baseUrl + encoded_args
+
+        encoded_data = json.dumps(jsonBody).encode('utf-8')
+        headers_dict = self._configuration.headers
+        # our content does not get recognized as json, so we set the header explicitly
+        headers_dict['Content-Type'] = 'application/json'
+        
+        # send the request
+        response = self._pm.request(
+            "PUT",
+            urlString,
+            body=encoded_data,
+            headers=headers_dict
+        )
+                
+        # reset the header as other requests cannot set it themselves otherwise
+        del headers_dict['Content-Type']
+        
+        if response.status > 399:
+            if not 'reason' in response.headers.keys():
+                raise Exception("Unknown Error fetching resource: HttpStatus: {};\n Message: {}\n Header of response: {}".format(response.status, response.data, response.headers))
+            else:
+                raise Exception("Could not get resource. Reason: {}".format(response.headers['reason']))
+        else:
+           provenanceInfoItem = json.loads(response.data.decode('utf-8'))
+           return provenanceInfoItem
+       
+        
+    # retrieve provenance report for network
+    def getProvenanceForNetwork(self, uuid):
+        self.checkSyncStatus()
+        baseUrl = "{}/networks/{}/prov".format(
+                    self._configuration.url,
+                    uuid
+                    )
+        response = self._pm.request("GET", baseUrl,
+                                   headers = self._configuration.headers)
+        if response.status > 399:
+            if response.headers['reason'] == None:
+                raise Exception("Unknown Error fetching resource: HttpStatus: {}; Header of response: {}".format(response.status, response.headers))
+            else:
+                raise Exception("Could not get resource. Reason: {}".format(response.headers['reason']))
+        else:
+            return json.loads(response.data.decode("utf-8"))
+        
\ No newline at end of file
